#ifndef SKETCH_H

#define SKETCH_H

#include <graphics.h>

#include "macro_be.h"

#ifndef max
#define max(a, b)	((a > b) ? a : b)
#endif  max

#define GRAPH_PATH ""

#define DIGIT_WIDTH     textwidth("8")
#define DIGIT_HEIGHT    textheight("8")
#define TEXT_WIDTH	textwidth("M")
#define TEXT_HEIGHT	textheight("M")

typedef struct graph_env {
	   struct pointtype max;
       } graph_env_t;

typedef struct layout {
			int bkcolor;
			int color;
			int scale_ps;
		     double scale_lb;
		     double scale_ub;
	   struct pointtype title_pos;
		       char title_str[80];
			int scale_dgt;
	   struct pointtype scale_pos;
	   struct pointtype count_prmpt_pos;
		       char count_str[20];
	   struct pointtype count_pos;
	   struct pointtype value_prmpt_pos;
		       char value_str[20];
	   struct pointtype value_pos;
	   struct pointtype status_pos;
	   struct pointtype left_top;
	   struct pointtype right_bottom;
	} layout_t;

#define Init_graph(gd) \
	MACRO_BEGIN \
	   int graphdriver = DETECT, graphmode, grapherror, errorcode; \
	   \
	   errorcode = registerbgidriver(EGAVGA_driver); \
	   if (errorcode < 0) { \
	      printf("Graphics error: %s\n", grapherrormsg(errorcode)); \
	      printf("Press any key to halt:"); \
	      getch(); \
	      exit(1); /* terminate with an error code */ \
	   } \
	   errorcode = registerbgidriver(CGA_driver); \
	   if (errorcode < 0) { \
	      printf("Graphics error: %s\n", grapherrormsg(errorcode)); \
	      printf("Press any key to halt:"); \
	      getch(); \
	      exit(1); /* terminate with an error code */ \
	   } \
	   errorcode = registerbgidriver(Herc_driver); \
	   if (errorcode < 0) { \
	      printf("Graphics error: %s\n", grapherrormsg(errorcode)); \
	      printf("Press any key to halt:"); \
	      getch(); \
	      exit(1); /* terminate with an error code */ \
	   } \
	   initgraph(&graphdriver, &graphmode, GRAPH_PATH); \
	   if ((grapherror = graphresult()) < 0) { \
	      fprintf(stderr, "initgraph error : %s.\n", \
			      grapherrormsg(grapherror)); \
	      exit(1); \
	   } \
	   gd = graphdriver; \
	MACRO_END

#define Close_graph \
	MACRO_BEGIN \
	   closegraph(); \
	MACRO_END

#define Decl_plot_vars \
      int _Base_x,  _Base_y; \
      int _Min_y,   _Max_y; \
      int _Plot_x,  _Plot_y, _Plot_xx, _Plot_yy; \
      int _Prev_x,  _Prev_y; \
      int _Curnt_x, _Curnt_y; \
      int _Prev_x_,  _Prev_y_; \
      int _Curnt_x_, _Curnt_y_; \
      int _Plot_x_,  _Plot_y_, _Plot_xx_, _Plot_yy_; \
      int _Count_x, _Count_y, _Count_xx, _Count_yy; \
      int _Value_x, _Value_y, _Value_xx, _Value_yy; \
      int _Status_x, _Status_y, _Status_xx, _Status_yy; \
   double _Xpoints, _Ypoints; \
   double _Ub, _Lb; \
     char _Count_s[8], _Value_s[16]; \
   double _Ii, _Range; \

#define Init_plot_env(layout, range) \
	MACRO_BEGIN \
	   _Base_x = layout.left_top.x; \
	   _Base_y = layout.right_bottom.y; \
	   _Min_y  = layout.left_top.y; \
	   _Max_y  = layout.right_bottom.y; \
	   _Xpoints = (double) (layout.right_bottom.x - layout.left_top.x); \
	   _Ypoints = (double) (layout.right_bottom.y - layout.left_top.y); \
	   _Range = range; \
	   _Ub = layout.scale_ub; \
	   _Lb = layout.scale_lb; \
	   \
	   setfillstyle(EMPTY_FILL, 0); \
	   _Count_x  = layout.count_pos.x; \
	   _Count_y  = layout.count_pos.y; \
	   _Count_xx = _Count_x + 6 * DIGIT_WIDTH; \
	   _Count_yy = _Count_y + DIGIT_HEIGHT; \
	   _Value_x  = layout.value_pos.x; \
	   _Value_y  = layout.value_pos.y; \
	   _Value_xx = _Value_x + 10 * DIGIT_WIDTH; \
	   _Value_yy = _Value_y + DIGIT_HEIGHT; \
	   _Status_x = layout.status_pos.x; \
	   _Status_y = layout.status_pos.y; \
	   _Status_xx = _Status_x + 20 * TEXT_WIDTH; \
	   _Status_yy = _Status_y + TEXT_HEIGHT; \
	MACRO_END

#define Plot_variation(i, steps, value) \
	MACRO_BEGIN \
	    _Plot_x = _Prev_x = _Curnt_x; \
	    _Plot_y = _Prev_y = _Curnt_y; \
	   _Ii = i; \
	   _Plot_xx = _Curnt_x = _Base_x + (int) (_Ii * _Xpoints / _Range); \
	   _Plot_yy = _Curnt_y = _Base_y - (int) ((value - _Lb) / \
				(_Ub - _Lb) * _Ypoints); \
	   if (i) { \
	      if (_Max_y < _Prev_y  || _Prev_y  < _Min_y) { \
		 _Plot_y = (_Max_y < _Prev_y) ? _Max_y : _Min_y; \
		 if (_Curnt_y == _Prev_y) \
		    _Plot_x = _Prev_x + steps / _Range; \
		 else \
		    _Plot_x = (_Plot_y - _Prev_y) * (_Curnt_x - _Prev_x) / \
			      (_Curnt_y - _Prev_y) + _Prev_x; \
	      } \
	      if (_Max_y < _Curnt_y || _Curnt_y < _Min_y) { \
		 _Plot_yy = (_Max_y < _Curnt_y) ? _Max_y : _Min_y; \
		 if (_Curnt_y == _Prev_y) \
		    _Plot_xx = _Prev_x + steps / _Range; \
		 else \
		    _Plot_xx = (_Plot_yy - _Prev_y) * (_Curnt_x - _Prev_x) / \
			    (_Curnt_y - _Prev_y) + _Prev_x; \
	      } \
	      line(_Plot_x, _Plot_y, _Plot_xx, _Plot_yy); \
		/* putpixel(_Plot_xx, _Plot_yy, WHITE);*/ \
	    } \
	MACRO_END

#define _Plot_variation(steps, prev_i, i, prev_value, value) \
	MACRO_BEGIN \
	   _Ii = prev_i; \
	   _Plot_x_ = _Prev_x_ = _Base_x + (int) (_Ii * _Xpoints / _Range); \
	   _Plot_y_ = _Prev_y_ = _Base_y - (int) ((prev_value - _Lb) / \
				(_Ub - _Lb) * _Ypoints); \
	   _Ii = i; \
	   _Plot_xx_ = _Curnt_x_ = _Base_x + (int) (_Ii * _Xpoints / _Range); \
	   _Plot_yy_ = _Curnt_y_ = _Base_y - (int) ((value - _Lb) / \
				(_Ub - _Lb) * _Ypoints); \
	   if (_Max_y < _Prev_y_  || _Prev_y_  < _Min_y) { \
	      _Plot_y_ = (_Max_y < _Prev_y_) ? _Max_y : _Min_y; \
	      if (_Curnt_y_ == _Prev_y_) \
		 _Plot_x_ = _Prev_x_ + steps / _Range; \
	      else \
		 _Plot_x_ = (_Plot_y_ - _Prev_y_) * (_Curnt_x_ - _Prev_x_) / \
			   (_Curnt_y_ - _Prev_y_) + _Prev_x_; \
	   } \
	   if (_Max_y < _Curnt_y_ || _Curnt_y_ < _Min_y) { \
	      _Plot_yy_ = (_Max_y < _Curnt_y_) ? _Max_y : _Min_y; \
	      if (_Curnt_y_ == _Prev_y_) \
		 _Plot_xx_ = _Prev_x_ + steps / _Range; \
	      else \
		 _Plot_xx_ = (_Plot_yy_ - _Prev_y_) * (_Curnt_x_ - _Prev_x_) / \
			    (_Curnt_y_ - _Prev_y_) + _Prev_x_; \
	    /*line(_Plot_x_, _Plot_y_, _Plot_xx_, _Plot_yy_);*/ \
	    /* putpixel(_Plot_xx_, _Plot_yy_, WHITE); */ \
	    } \
	MACRO_END


	    /* itoa(i, _Count_s, 10); \
	    if (sprintf(_Value_s, "%9.6lf", value) == EOF) { \
	       fprintf(stderr, "Coversion in sprintf failed\n"); \
	       Close_graph \
	       exit(1); \
	    } \
	    bar(_Count_x, _Count_y, _Count_xx, _Count_yy); \
	    bar(_Value_x, _Value_y, _Value_xx, _Value_yy); \
	    outtextxy(_Count_x, _Count_y, _Count_s); \
	    outtextxy(_Value_x, _Value_y, _Value_s); \
	MACRO_END */

graph_env_t *get_graph_env(graph_env_t*);
void setup_diagram_layout(layout_t);

#endif SKETCH_H