#include <stdio.h>
#include <stdlib.h>
#include <time.h>
#include <conio.h>

#include "debug.h"
#include "boolean.h"
#include "macro_be.h"
#include "sketch.h"
#include "keyin.h"
#include "qc_ad.h"

/*
 *	Constant String
 */
#define TEST_ADVANCE_TITLE	"Testing AD of Advance"


/*
 * 	Constant Value for graphics
 */
#define PROMPT_TEXT	1
#define SCALE_DIGITS	7
#define STATUS_DIGITS	1
#define BIAS		5
#define FILE_SAVING	"File is being saved"

/*
 *	Constants for work mode
 */
#define BATCH		0
#define DIAMETER	1

/*
 *	Data Pool for storing sample AD data
 */
typedef struct {
	   unsigned char hi;
	   unsigned char lo;
	} ad_value_t;

typedef struct {
	   int 	  scrn_i;
	   double cv;
	} win_save_t;


/*
 * Constants for testing
 */
#define THRESHOLD	2048		/* middle value of full range */
#define STEP_WIDE 	2
#define MAX_TOLERANCE	5

/*
 * configuration for tesing
 */
typedef struct config {
	   unsigned char fn[32];
	   unsigned char channel;
	   unsigned int  bits;
	   unsigned int  opmode;
	   unsigned int  steps;
	   unsigned int  t_delay;
	   unsigned int  workmode;
	   unsigned int  count;
	   long          i_freq;
	   long    	 sample_rate;
	} config_t;


boolean_t get_config(config_t *config, int no, i_pat_t tbl[])
{
   i_pat_t *ptr;

   ptr = tbl;
   config->channel = ptr->ret.alts;

   ptr = tbl + 1;
   config->opmode = (ptr->ret.alts == 0) ? UNIPOLAR : BIPOLAR;

   ptr = tbl + 2;
   config->count = (1 << (ptr->ret.alts+7));  /* 128 ---> */

   ptr = tbl + 3;
   switch (ptr->ret.alts) {
      case 0 : config->workmode = BATCH;
	       break;
      case 1 : config->workmode = DIAMETER;
	       break;
   }

   ptr = tbl + 4;
   config->i_freq = _atold(ptr->ret.str) * 1000L;
   if (config->i_freq == 0) return FALSE;

   return TRUE;
}

layout_t *get_diagram_layout(layout_t *layout, int gd, graph_env_t env,
			     int scale_ps, char *prompt1, char *prompt2)
{
   double ub, lb;

   switch (gd) {
      case     MCGA :
      case  EGAMONO :
      case HERCMONO : layout->bkcolor = 0;
		      layout->color = 15;
		      break;
	    default : layout->bkcolor = BLUE;
		      layout->color = YELLOW;
		      break;
   }
   strcpy(layout->title_str, TEST_ADVANCE_TITLE);
   layout->title_pos.x = (env.max.x -
			  TEXT_WIDTH * strlen(layout->title_str)) / 2;
   layout->title_pos.y = 0;

   layout->scale_pos.x = 0;
   layout->scale_dgt   = SCALE_DIGITS;

   layout->left_top.x     = layout->scale_pos.x + layout->scale_dgt * DIGIT_WIDTH + BIAS;
   layout->left_top.y     = layout->title_pos.y + TEXT_HEIGHT + BIAS;
   layout->right_bottom.x = env.max.x;

   strcpy(layout->count_str, prompt1);
   layout->count_prmpt_pos.x = layout->left_top.x;
   layout->count_prmpt_pos.y = env.max.y - max(TEXT_HEIGHT, DIGIT_HEIGHT);
   layout->count_pos.x       = layout->count_prmpt_pos.x + TEXT_WIDTH *
			       strlen(layout->count_str);
   layout->count_pos.y       = layout->count_prmpt_pos.y;
   layout->right_bottom.y    = layout->count_pos.y - BIAS;

   strcpy(layout->value_str, prompt2);
   layout->value_prmpt_pos.x = env.max.x / 2 - env.max.x / 5;
   layout->value_prmpt_pos.y = layout->count_pos.y;
   layout->value_pos.x       = layout->value_prmpt_pos.x + TEXT_WIDTH *
			       strlen(layout->value_str);
   layout->value_pos.y       = layout->value_prmpt_pos.y;

   layout->status_pos.x      = env.max.x / 2 + env.max.x / 16;
   layout->status_pos.y      = layout->count_pos.y;

   layout->scale_pos.y       = layout->left_top.y - DIGIT_HEIGHT / 2;
   layout->scale_ps          = scale_ps;

   ub = +10.0;
   lb = -10.0;
   layout->scale_lb = lb;
   layout->scale_ub = ub;

   return layout;
}

#define PRINT_VALUES(value1, value2) \
	MACRO_BEGIN \
	   ltoa(value1, _Count_s, 10); \
	   ltoa(value2, _Value_s, 10); \
	   bar(_Count_x, _Count_y, _Count_xx, _Count_yy); \
	   bar(_Value_x, _Value_y, _Value_xx, _Value_yy); \
	   outtextxy(_Count_x, _Count_y, _Count_s); \
	   outtextxy(_Value_x, _Value_y, _Value_s); \
	MACRO_END

/*
 * 	forward reference for sample_and_display()
 */
void start_sample(ad_value_t far data_pool[], unsigned char channel,
		  unsigned t_delay, unsigned count);

void b_sample_and_display(config_t *config)
{
   Decl_plot_vars
   graph_env_t env;
   layout_t    layout;
   int	       gd, scrn_width;

   unsigned opmode    = config->opmode,
	    bits      = config->bits,
	    count     = config->count;
   long	    i_freq    = config->i_freq;

   int            dp_i;
   ad_value_t far *datapool;

   int		  scrn_i, scrn_size;

   int            front_win_i = 0, rear_win_i = 0, next_win_i, win_size;
   win_save_t	  *save_win;
   boolean_t	  remove = FALSE;

   double         cv;
   unsigned int   ad_value, hi, lo;

   long		  cycles;
   int		  x, pre_x = 0;

#ifndef DEBUG
   if ((datapool = (ad_value_t *) malloc(count * sizeof(ad_value_t))) == NULL) {
      printf("Not enough memory to allocate buffer\n");
      exit(1);  /* terminate program if out of memory */
   }

   cprintf("starting sampling ...\r\n");
   start_sample(datapool, config->channel, config->t_delay, count);
   cprintf("ending sampling, strike any key when ready\r\n");
   getch();
#endif DEBUG

   Init_graph(gd);

   get_graph_env(&env);
   get_diagram_layout(&layout, gd, env, 8, "Cycles = ", "Rate = ");
   scrn_width = layout.right_bottom.x - layout.left_top.x;
   setup_diagram_layout(layout);
   scrn_size = (count < scrn_width) ? count : scrn_width;
   Init_plot_env(layout, scrn_size);
   win_size = scrn_size;

   if ((save_win = (win_save_t *) malloc(win_size * sizeof(win_save_t))) == NULL) {
      printf("Not enough memory to allocate buffer\n");
      exit(1);  /* terminate program if out of memory */
   }

#ifdef DEBUG
   randomize();
#endif DEBUG

   for (cycles = dp_i = scrn_i = 0; dp_i < count; dp_i++, scrn_i++) {

      if (rear_win_i >= win_size) {
	 rear_win_i = 0;
	 remove = TRUE;
      }

      if (remove) {
	 next_win_i = (front_win_i + 1) % win_size;
	 if (save_win[next_win_i].scrn_i != 0) {
	    _Plot_variation(1,
		    save_win[front_win_i].scrn_i, save_win[next_win_i].scrn_i,
		    save_win[front_win_i].cv, save_win[next_win_i].cv);
	    setcolor(layout.bkcolor);
	    line(_Plot_x_, _Plot_y_, _Plot_xx_, _Plot_yy_);
	 }
	 front_win_i = next_win_i;
      }

#ifdef DEBUG
      hi = rand() % 16;
      lo = rand() % 256;
#else DEBUG
      hi = (unsigned int)datapool[dp_i].hi;
      lo = (unsigned int)datapool[dp_i].lo;
#endif DEBUG
      ad_value = (hi << 8) | lo;

      x = (ad_value > THRESHOLD);
      if (x && !pre_x) cycles++;
      pre_x = x;

      AD_Ideal_Voltage(cv, ad_value, opmode, AD_RANGE)

      if (scrn_i > scrn_size) scrn_i = 0;

      setcolor(layout.color);
      Plot_variation(scrn_i, 1, cv);

      save_win[rear_win_i].scrn_i = scrn_i;
      save_win[rear_win_i].cv = cv;
      rear_win_i++;

      if (kbhit()) if (getch() == KEYESC) break;
   }
   if (cycles != 0) config->sample_rate = (long)count * i_freq / cycles;

   PRINT_VALUES(cycles, config->sample_rate);

   getch();

   Close_graph

   free(save_win);
#ifndef DEBUG
   free(datapool);
#endif DEBUG
}

void b1_sample_and_display(config_t *config)
{
   Decl_plot_vars
   graph_env_t env;
   layout_t    layout;
   int	       gd, scrn_width;

   unsigned int maxv, peakx;

   int            dp_i;
   ad_value_t far *datapool;

   unsigned opmode      = config->opmode,
	    bits        = config->bits,
	    steps       = config->steps,
	    count       = config->count,
	    half_count  = count >> 1;
   long	    i_freq      = config->i_freq,
	    sample_rate	= config->sample_rate;

   int	    scrn_i, scrn_size;

   int 	    draw_scrn = 0;

   int        front_win_i = 0, rear_win_i = 0 , next_win_i, win_size;
   win_save_t *save_win;
   boolean_t  remove = FALSE;

   double       cv;
   unsigned int ad_value, hi, lo;

   long		rate, cycles;
   int		x, pre_x = 0;

   int 		out = 0;

#ifndef DEBUG
   if ((datapool = (ad_value_t *) malloc(count * sizeof(ad_value_t))) == NULL) {
      printf("Not enough memory to allocate buffer\n");
      exit(1);  /* terminate program if out of memory */
   }
#endif DEBUG

   Init_graph(gd);

   get_graph_env(&env);
   get_diagram_layout(&layout, gd, env, 8, "Cycles = ", "Input Freg. = ");
   scrn_width = layout.right_bottom.x - layout.left_top.x;
   setup_diagram_layout(layout);
   scrn_size = (count < scrn_width) ? count : scrn_width;
   Init_plot_env(layout, scrn_size);
   win_size = scrn_size;

   if ((save_win = (win_save_t *) malloc(scrn_size * sizeof(win_save_t))) == NULL) {
      printf("Not enough memory to allocate buffer\n");
      exit(1);  /* terminate program if out of memory */
   }

#ifdef DEBUG
   randomize();
#endif DEBUG

   while (TRUE) {
#ifndef DEBUG
      start_sample(datapool, config->channel, config->t_delay, count);
#endif DEBUG

      for (cycles = 0, maxv = 0L, peakx = dp_i = 0; dp_i < count; dp_i++) {
	 hi = (unsigned int)datapool[dp_i].hi;
	 lo = (unsigned int)datapool[dp_i].lo;
	 ad_value = (hi << 8) | lo;

	 if (dp_i < half_count) {		/* for locking phase */
	    if (ad_value > maxv) {
	       maxv = ad_value;
	       peakx = dp_i;
	    }
	 }

	 x = (ad_value > THRESHOLD);
	 if (x && !pre_x) cycles++;
	 pre_x = x;
      }

      for (dp_i = peakx, scrn_i = 0;
	   scrn_i < scrn_size && dp_i < count; dp_i+=1, scrn_i+=steps) {

	 if (rear_win_i >= win_size) {
	    rear_win_i = 0;
	    remove = TRUE;
	 }

	 if (remove) {
	    next_win_i = (front_win_i + 1) % win_size;
	    if (save_win[next_win_i].scrn_i != 0) {
	       _Plot_variation(steps,
		       save_win[front_win_i].scrn_i, save_win[next_win_i].scrn_i,
		       save_win[front_win_i].cv, save_win[next_win_i].cv);
	       setcolor(layout.bkcolor);
	       line(_Plot_x_, _Plot_y_, _Plot_xx_, _Plot_yy_);
	    }
	    front_win_i = next_win_i;
	 }

#ifdef DEBUG
	 hi = rand() % 16;
	 lo = rand() % 256;
#else DEBUG
	 hi = (unsigned int)datapool[dp_i].hi;
	 lo = (unsigned int)datapool[dp_i].lo;
#endif DEBUG
	 ad_value = (hi << 8) | lo;

	 AD_Ideal_Voltage(cv, ad_value, opmode, AD_RANGE)

	 if (scrn_i > scrn_size) scrn_i = 0;

	 setcolor(layout.color);
	 Plot_variation(scrn_i, steps, cv);

	 save_win[rear_win_i].scrn_i = scrn_i;
	 save_win[rear_win_i].cv = cv;
	 rear_win_i++;

	 if (kbhit()) {
	    char c = getch();

	    if (c == KEYESC) { out = 1; break; }
	    /*if (c == 'w') if (steps < 30) steps += STEP_WIDE;
	    if (c == 'n') if (steps >= 3) steps -= STEP_WIDE;*/
	 }
      }
      if (!remove) remove = TRUE;

      i_freq = sample_rate * cycles / count;

      PRINT_VALUES(cycles, i_freq);

      if (out) break;

      if (draw_scrn++ > MAX_TOLERANCE) {
	 setup_diagram_layout(layout);
	 draw_scrn = 0;
      }
   } /* while */

   Close_graph;

   free(save_win);
#ifndef DEBUG
   free(datapool);
#endif DEBUG
}

void start_sample(ad_value_t far datapool[], unsigned char channel,
		  unsigned t_delay, unsigned count)
{
   int port = ADVANCE_BASE_PORT;
   unsigned char hi, lo;
   unsigned ad_value, i = 0;
   double cv;

   asm 	push	es

   asm 	push	ax
   asm	push	bx
   asm  push	cx
   asm 	push 	dx
   asm	push	di

   asm	mov	cx,count
   asm 	les	di,datapool

   asm	cli
next:
   /* outportb(port+2, channel) */
   asm 	mov	dx,port;
   asm 	inc     dx
   asm	inc	dx
   asm 	mov	al,0
   asm  out	dx,al
//next:
   /* outportb(port+3, 0xFF) */
   asm	mov	dx,port
   asm 	add	dx,3
   asm 	mov	al,0ffh
   asm  out	dx,al

   /* do { hi = inportb(port+3); delay(5); } while (hi >= 0x80); */
again:
   asm	in	al,dx
   asm 	cmp	al,80h
   asm  ja	again
   asm	stosb

   /* lo = inportb(port+2) */
   asm 	dec	dx
   asm 	in	al,dx
   asm  stosb

/*   asm  push	cx
   asm  mov	cx,1000

   asm  loop	here
   asm 	pop	cx
*/
   asm	loop	next

   asm	sti

   asm	pop	di
   asm 	pop	dx
   asm  pop	cx
   asm	pop	bx
   asm 	pop	ax

   asm	pop	es
}

void main()
{
   config_t config = {		/* configuration data for testing */
	    "", 		/* fn */
	    0,			/* channel */
	    AD_BITS,		/* bits */
	    BIPOLAR, 		/* opmode */
	    1,			/* steps */
	    1,			/* t_delay */
	    BATCH,		/* work mode */
	    8192,		/* count */
	    1000,		/* i_freq */
	    164500            	/* sample_rate */
   };

   i_pat_t input_tbl[] = {
	   {
	      GetAlts,
	      { { 10,  8 }, "   Channel        : " },
	      { 29, 8 },
	      { 16, { "1", "2", "3", "4", "5", "6", "7", "8",
		      "9", "10", "11", "12", "13", "14", "15", "16" } },
	      { 0 },
	      NULL
	   },
	   {
	      GetAlts,
	      { { 10,  10 }, "     Polar       : " },
	      { 29, 10 },
	      { 2, { "UNIPOLAR", "BIOPOLAR"} },
	      { 1 },
	      NULL
	   },
	   {
	      GetAlts,
	      { { 10,  12 }, "     Count       : " },
	      { 29, 12 },
	      { 7, { "128", "256", "512", "1024", "2048", "4096", "8192"
		   } },
	      { 6 },
	      NULL
	   },
	   {
	      GetAlts,
	      { { 10,  14 }, " Work Mode       : " },
	      { 29, 14 },
	      { 2, {"BATCH", "DIAMETER" } },
	      { 1 },
	      NULL
	   },
	   {
	      GetStr,
	      { { 10,  16 }, "Input Freq.(KHZ) : " },
	      { 29, 16 },
	      { 9, {"20.22"} },
	      { 0 },
	      NULL
	   }
   };

   key_code_t rtn;

#define select(pat_tbl) \
	drive_input(sizeof(pat_tbl) / sizeof(i_pat_t), pat_tbl)

   do {
      setup_i_layout(TEST_ADVANCE_TITLE, Author_Date);
      rtn = select(input_tbl);
      clrscr();
      if (rtn == KEYEXIT) break;

      if (!get_config(&config, sizeof(input_tbl) / sizeof(i_pat_t), input_tbl))
	 continue;

      switch (config.workmode) {
	 case     BATCH : b_sample_and_display(&config);
			  break;
	 case  DIAMETER : b1_sample_and_display(&config);
			  break;
      }

   } while (TRUE);
   clrscr();

   exit(0);
}